@@ -3,18 +3,17 @@ This module contains the ToolsFactory class for creating agent tools.
 """
 
 import inspect
-import re
 import importlib
 import os
 import asyncio
-
 from typing import Callable, List, Dict, Any, Optional, Union
+
+from retrying import retry
 from pydantic import BaseModel, Field
 
 from llama_index.core.tools import FunctionTool
 from llama_index.indices.managed.vectara import VectaraIndex
 from llama_index.core.utilities.sql_wrapper import SQLDatabase
-from llama_index.core.tools.types import ToolOutput
 
 from .types import ToolType
 from .tools_catalog import ToolsCatalog, get_bad_topics
@@ -25,6 +24,7 @@ from .tool_utils import (
     create_tool_from_dynamic_function,
     build_filter_string,
     VectaraTool,
+    create_human_readable_output,
 )
 
 LI_packages = {
@@ -65,6 +65,18 @@ LI_packages = {
 }
 
 
+@retry(stop_max_attempt_number=3, wait_exponential_multiplier=1000, wait_exponential_max=10000)
+def _query_with_retry(vectara_query_engine, query):
+    """Execute Vectara query with automatic retry on timeout/failure."""
+    return vectara_query_engine.query(query)
+
+
+@retry(stop_max_attempt_number=3, wait_exponential_multiplier=1000, wait_exponential_max=10000)
+def _retrieve_with_retry(vectara_retriever, query):
+    """Execute Vectara retrieve with automatic retry on timeout/failure."""
+    return vectara_retriever.retrieve(query)
+
+
 class VectaraToolFactory:
     """
     A factory class for creating Vectara RAG tools.
@@ -74,20 +86,16 @@ class VectaraToolFactory:
         self,
         vectara_corpus_key: str = str(os.environ.get("VECTARA_CORPUS_KEY", "")),
         vectara_api_key: str = str(os.environ.get("VECTARA_API_KEY", "")),
-        compact_docstring: bool = False,
     ) -> None:
         """
         Initialize the VectaraToolFactory
         Args:
             vectara_corpus_key (str): The Vectara corpus key (or comma separated list of keys).
             vectara_api_key (str): The Vectara API key.
-            compact_docstring (bool): Whether to use a compact docstring format for tools
-              This is useful if OpenAI complains on the 1024 token limit.
         """
         self.vectara_corpus_key = vectara_corpus_key
         self.vectara_api_key = vectara_api_key
         self.num_corpora = len(vectara_corpus_key.split(","))
-        self.compact_docstring = compact_docstring
 
     def create_search_tool(
         self,
@@ -116,6 +124,7 @@ class VectaraToolFactory:
         verbose: bool = False,
         vectara_base_url: str = "https://api.vectara.io",
         vectara_verify_ssl: bool = True,
+        vhc_eligible: bool = True,
     ) -> VectaraTool:
         """
         Creates a Vectara search/retrieval tool
@@ -168,9 +177,10 @@ class VectaraToolFactory:
             vectara_base_url=vectara_base_url,
             vectara_verify_ssl=vectara_verify_ssl,
         )
+        vectara.vectara_api_timeout = 10
 
         # Dynamically generate the search function
-        def search_function(*args: Any, **kwargs: Any) -> ToolOutput:
+        def search_function(*args: Any, **kwargs: Any) -> list[dict]:
             """
             Dynamically generated function for semantic search Vectara.
             """
@@ -192,12 +202,11 @@ class VectaraToolFactory:
                     kwargs, tool_args_type, fixed_filter
                 )
             except ValueError as e:
-                return ToolOutput(
-                    tool_name=search_function.__name__,
-                    content=str(e),
-                    raw_input={"args": args, "kwargs": kwargs},
-                    raw_output={"response": str(e)},
+                msg = (
+                    f"Building filter string failed in search tool due to invalid input or configuration ({e}). "
+                    "Please verify the input arguments and ensure they meet the expected format or conditions."
                 )
+                return [{"text": msg, "metadata": {"args": args, "kwargs": kwargs}}]
 
             vectara_retriever = vectara.as_retriever(
                 summary_enabled=False,
@@ -224,16 +233,11 @@ class VectaraToolFactory:
                 x_source_str="vectara-agentic",
                 verbose=verbose,
             )
-            response = vectara_retriever.retrieve(query)
+            response = _retrieve_with_retry(vectara_retriever, query)
 
             if len(response) == 0:
                 msg = "Vectara Tool failed to retrieve any results for the query."
-                return ToolOutput(
-                    tool_name=search_function.__name__,
-                    content=msg,
-                    raw_input={"args": args, "kwargs": kwargs},
-                    raw_output={"response": msg},
-                )
+                return [{"text": msg, "metadata": {"args": args, "kwargs": kwargs}}]
             unique_ids = set()
             docs = []
             doc_matches = {}
@@ -244,7 +248,8 @@ class VectaraToolFactory:
                 unique_ids.add(doc.id_)
                 doc_matches[doc.id_] = [doc.node.get_content()]
                 docs.append((doc.id_, doc.metadata))
-            tool_output = "Matching documents:\n"
+
+            res = []
             if summarize:
                 summaries_dict = asyncio.run(
                     summarize_documents(
@@ -254,35 +259,45 @@ class VectaraToolFactory:
                         doc_ids=list(unique_ids),
                     )
                 )
-                for doc_id, metadata in docs:
-                    summary = summaries_dict.get(doc_id, "")
-                    matching_text = "\n".join(
-                        f"<match {i}>{piece}</match {i}>"
-                        for i, piece in enumerate(doc_matches[doc_id], start=1)
-                    )
-                    tool_output += (
-                        f"document_id: '{doc_id}'\nmetadata: '{metadata}'\n"
-                        f"matching texts: '{matching_text}'\n"
-                        f"summary: '{summary}'\n\n"
-                    )
             else:
-                for doc_id, metadata in docs:
-                    matching_text = "\n".join(
-                        f"<match {i}>{piece}</match {i}>"
-                        for i, piece in enumerate(doc_matches[doc_id], start=1)
-                    )
-                    tool_output += (
-                        f"document_id: '{doc_id}'\nmetadata: '{metadata}'\n"
-                        f"matching texts: '{matching_text}'\n"
-                    )
+                summaries_dict = {}
 
-            out = ToolOutput(
-                tool_name=search_function.__name__,
-                content=tool_output,
-                raw_input={"args": args, "kwargs": kwargs},
-                raw_output=response,
-            )
-            return out
+            for doc_id, metadata in docs:
+                res.append(
+                    {
+                        "text": summaries_dict.get(doc_id, "") if summarize else "",
+                        "metadata": {
+                            "document_id": doc_id,
+                            "metadata": metadata,
+                            "matching_text": doc_matches[doc_id],
+                        },
+                    }
+                )
+
+            # Create human-readable output using sequential format
+            def format_search_results(results):
+                if not results:
+                    return "No search results found"
+
+                # Create a sequential view for human reading
+                formatted_results = []
+                for i, result in enumerate(results, 1):
+                    result_str = f"**Result #{i}**\n"
+                    result_str += f"Document ID: {result['metadata']['document_id']}\n"
+                    if summarize and result["text"]:
+                        result_str += f"Summary: {result['text']}\n"
+
+                    # Add all matching text if available
+                    matches = result["metadata"]["matching_text"]
+                    if matches:
+                        result_str += "".join(
+                            f"Match #{inx} Text: {match}\n"
+                            for inx, match in enumerate(matches, 1)
+                        )
+                    formatted_results.append(result_str)
+                return "\n".join(formatted_results)
+
+            return create_human_readable_output(res, format_search_results)
 
         class SearchToolBaseParams(BaseModel):
             """Model for the base parameters of the search tool."""
@@ -307,7 +322,7 @@ class VectaraToolFactory:
                 description="The search query to perform, in the form of a question.",
             )
             top_k: int = Field(
-                10, description="The number of top documents to retrieve."
+                default=10, description="The number of top documents to retrieve."
             )
 
         search_tool_extra_desc = (
@@ -326,8 +341,8 @@ class VectaraToolFactory:
                 else SearchToolBaseParamsWithoutSummarize
             ),
             tool_args_schema,
-            compact_docstring=self.compact_docstring,
             return_direct=return_direct,
+            vhc_eligible=vhc_eligible,
         )
         return tool
 
@@ -362,12 +377,17 @@ class VectaraToolFactory:
         frequency_penalty: Optional[float] = None,
         presence_penalty: Optional[float] = None,
         include_citations: bool = True,
+        citation_pattern: str = None,
+        citation_url_pattern: str = "{doc.url}",
+        citation_text_pattern: str = "{doc.title}",
         save_history: bool = False,
         fcs_threshold: float = 0.0,
         return_direct: bool = False,
+        return_human_readable_output: bool = False,
         verbose: bool = False,
         vectara_base_url: str = "https://api.vectara.io",
         vectara_verify_ssl: bool = True,
+        vhc_eligible: bool = True,
     ) -> VectaraTool:
         """
         Creates a RAG (Retrieve and Generate) tool.
@@ -415,10 +435,18 @@ class VectaraToolFactory:
                 higher values increasing the diversity of topics.
             include_citations (bool, optional): Whether to include citations in the response.
                 If True, uses markdown vectara citations that requires the Vectara scale plan.
+            citation_url_pattern (str, optional): The pattern for the citations in the response.
+                Default is "{doc.url}" which uses the document URL.
+                If include_citations is False, this parameter is ignored.
+                citation_pattern (str, optional): old name for citation_url_pattern. Deprecated.
+            citation_text_pattern (str, optional): The text pattern for citations in the response.
+                Default is "{doc.title}" which uses the title of the document.
+                If include_citations is False, this parameter is ignored.
             save_history (bool, optional): Whether to save the query in history.
             fcs_threshold (float, optional): A threshold for factual consistency.
                 If set above 0, the tool notifies the calling agent that it "cannot respond" if FCS is too low.
             return_direct (bool, optional): Whether the agent should return the tool's response directly.
+            return_human_readable_output (bool, optional): Whether to return the output in a human-readable format.
             verbose (bool, optional): Whether to print verbose output.
             vectara_base_url (str, optional): The base URL for the Vectara API.
             vectara_verify_ssl (bool, optional): Whether to verify SSL certificates for the Vectara API.
@@ -434,9 +462,11 @@ class VectaraToolFactory:
             vectara_base_url=vectara_base_url,
             vectara_verify_ssl=vectara_verify_ssl,
         )
+        vectara.vectara_api_timeout = 60
+        keys_to_ignore = ["lang", "offset", "len"]
 
         # Dynamically generate the RAG function
-        def rag_function(*args: Any, **kwargs: Any) -> ToolOutput:
+        def rag_function(*args: Any, **kwargs: Any) -> dict:
             """
             Dynamically generated function for RAG query with Vectara.
             """
@@ -452,12 +482,23 @@ class VectaraToolFactory:
                     kwargs, tool_args_type, fixed_filter
                 )
             except ValueError as e:
-                return ToolOutput(
-                    tool_name=rag_function.__name__,
-                    content=str(e),
-                    raw_input={"args": args, "kwargs": kwargs},
-                    raw_output={"response": str(e)},
+                msg = (
+                    f"Building filter string failed in rag tool. "
+                    f"Reason: {e}. Ensure that the input arguments match the expected "
+                    f"format and include all required fields. "
                 )
+                return {"text": msg, "metadata": {"args": args, "kwargs": kwargs}}
+
+            computed_citations_url_pattern = (
+                (
+                    citation_url_pattern
+                    if citation_url_pattern is not None
+                    else citation_pattern
+                )
+                if include_citations
+                else None
+            )
+            computed_citations_text_pattern = citation_text_pattern if include_citations else None
 
             vectara_query_engine = vectara.as_query_engine(
                 summary_enabled=True,
@@ -491,85 +532,101 @@ class VectaraToolFactory:
                 frequency_penalty=frequency_penalty,
                 presence_penalty=presence_penalty,
                 citations_style="markdown" if include_citations else None,
-                citations_url_pattern="{doc.url}" if include_citations else None,
+                citations_url_pattern=computed_citations_url_pattern,
+                citations_text_pattern=computed_citations_text_pattern,
                 save_history=save_history,
                 x_source_str="vectara-agentic",
                 verbose=verbose,
             )
-            response = vectara_query_engine.query(query)
+            response = _query_with_retry(vectara_query_engine, query)
 
             if len(response.source_nodes) == 0:
                 msg = (
                     "Tool failed to generate a response since no matches were found. "
                     "Please check the arguments and try again."
                 )
-                return ToolOutput(
-                    tool_name=rag_function.__name__,
-                    content=msg,
-                    raw_input={"args": args, "kwargs": kwargs},
-                    raw_output={"response": msg},
-                )
+                kwargs["query"] = query
+                return {"text": msg, "metadata": {"args": args, "kwargs": kwargs}}
             if str(response) == "None":
                 msg = "Tool failed to generate a response."
-                return ToolOutput(
-                    tool_name=rag_function.__name__,
-                    content=msg,
-                    raw_input={"args": args, "kwargs": kwargs},
-                    raw_output={"response": msg},
-                )
+                kwargs["query"] = query
+                return {"text": msg, "metadata": {"args": args, "kwargs": kwargs}}
 
-            # Extract citation metadata
-            pattern = r"\[(\d+)\]"
-            matches = re.findall(pattern, response.response)
-            citation_numbers = sorted(set(int(match) for match in matches))
-            citation_metadata = ""
-            keys_to_ignore = ["lang", "offset", "len"]
-            for citation_number in citation_numbers:
-                metadata = response.source_nodes[citation_number - 1].metadata
-                citation_metadata += (
-                    f"[{citation_number}]: "
-                    + "; ".join(
-                        [
-                            f"{k}='{v}'"
-                            for k, v in metadata.items()
-                            if k not in keys_to_ignore
-                        ]
-                    )
-                    + ".\n"
-                )
             fcs = 0.0
             fcs_str = response.metadata["fcs"] if "fcs" in response.metadata else "0.0"
             if fcs_str and is_float(fcs_str):
                 fcs = float(fcs_str)
                 if fcs < fcs_threshold:
                     msg = f"Could not answer the query due to suspected hallucination (fcs={fcs})."
-                    return ToolOutput(
-                        tool_name=rag_function.__name__,
-                        content=msg,
-                        raw_input={"args": args, "kwargs": kwargs},
-                        raw_output={"response": msg},
+                    return {
+                        "text": msg,
+                        "metadata": {"args": args, "kwargs": kwargs, "fcs": fcs},
+                    }
+
+            # Add source nodes to tool output
+            if ((not return_human_readable_output) and
+                (computed_citations_url_pattern is not None) and
+                (computed_citations_text_pattern is not None)):
+                response_text = str(response.response)
+                citation_metadata = []
+
+                # Converts a dictionary to an object with .<field> access
+                def to_obj(data):
+                    return type('obj', (object,), data)()
+
+                for source_node in response.source_nodes:
+                    node = source_node.node
+                    node_id = node.id_
+                    node_text = (
+                        node.text_resource.text if hasattr(node, 'text_resource')
+                        else getattr(node, 'text', '')
                     )
-            res = {
-                "response": response.response,
-                "references_metadata": citation_metadata,
-                "fcs_score": fcs,
-            }
-            if len(citation_metadata) > 0:
-                tool_output = f"""
-                    Response: '''{res['response']}'''
-                    fcs_score: {res['fcs_score']:.4f}
-                    References:
-                    {res['references_metadata']}
-                """
+                    node_metadata = getattr(node, 'metadata', {})
+                    for key in keys_to_ignore:
+                        if key in node_metadata:
+                            del node_metadata[key]
+
+                    try:
+                        template_data = {}
+
+                        doc_data = node_metadata.get('document', {})
+                        template_data['doc'] = to_obj(doc_data)
+
+                        part_data = {k: v for k, v in node_metadata.items() if k != 'document'}
+                        template_data['part'] = to_obj(part_data)
+
+                        formatted_citation_text = computed_citations_text_pattern.format(**template_data)
+                        formatted_citation_url = computed_citations_url_pattern.format(**template_data)
+                        expected_citation = f"[{formatted_citation_text}]({formatted_citation_url})"
+
+                        if expected_citation in response_text:
+                            citation_metadata.append({
+                                'doc_id': node_id,
+                                'text': node_text,
+                                'metadata': node_metadata,
+                                'score': getattr(node, 'score', None)
+                            })
+
+                    except Exception as e:
+                        if verbose:
+                            print(f"Could not format citation for search result {node_id}: {e}")
+                        continue
+
+                res = {"text": response.response, "citations": citation_metadata}
+                if fcs:
+                    res["fcs"] = fcs
             else:
-                tool_output = f"Response: '''{res['response']}'''"
-            out = ToolOutput(
-                tool_name=rag_function.__name__,
-                content=tool_output,
-                raw_input={"args": args, "kwargs": kwargs},
-                raw_output=res,
-            )
-            return out
+                res = {"text": response.response}
+
+            # Create human-readable output
+            if return_human_readable_output:
+                def format_rag_response(result):
+                    text = result["text"]
+                    return text
+
+                return create_human_readable_output(res, format_rag_response)
+
+            return res
 
         class RagToolBaseParams(BaseModel):
             """Model for the base parameters of the RAG tool."""
@@ -585,8 +642,8 @@ class VectaraToolFactory:
             tool_description,
             RagToolBaseParams,
             tool_args_schema,
-            compact_docstring=self.compact_docstring,
             return_direct=return_direct,
+            vhc_eligible=vhc_eligible,
         )
         return tool
 
@@ -600,7 +657,10 @@ class ToolsFactory:
         self.agent_config = agent_config
 
     def create_tool(
-        self, function: Callable, tool_type: ToolType = ToolType.QUERY
+        self,
+        function: Callable,
+        tool_type: ToolType = ToolType.QUERY,
+        vhc_eligible: bool = True,
     ) -> VectaraTool:
         """
         Create a tool from a function.
@@ -612,7 +672,9 @@ class ToolsFactory:
         Returns:
             VectaraTool: A VectaraTool object.
         """
-        return VectaraTool.from_defaults(tool_type=tool_type, fn=function)
+        return VectaraTool.from_defaults(
+            tool_type=tool_type, fn=function, vhc_eligible=vhc_eligible
+        )
 
     def get_llama_index_tools(
         self,
@@ -673,7 +735,7 @@ class ToolsFactory:
         """
         tc = ToolsCatalog(self.agent_config)
         return [
-            self.create_tool(tool)
+            self.create_tool(tool, vhc_eligible=True)
             for tool in [tc.summarize_text, tc.rephrase_text, tc.critique_text]
         ]
 
@@ -681,7 +743,7 @@ class ToolsFactory:
         """
         Create a list of guardrail tools to avoid controversial topics.
         """
-        return [self.create_tool(get_bad_topics)]
+        return [self.create_tool(get_bad_topics, vhc_eligible=False)]
 
     def financial_tools(self):
         """
@@ -722,7 +784,8 @@ class ToolsFactory:
             )
 
         return [
-            self.create_tool(tool) for tool in [summarize_legal_text, critique_as_judge]
+            self.create_tool(tool, vhc_eligible=False)
+            for tool in [summarize_legal_text, critique_as_judge]
         ]
 
     def database_tools(
@@ -797,6 +860,7 @@ class ToolsFactory:
                 fn=tool.fn,
                 async_fn=tool.async_fn,
                 metadata=tool.metadata,
+                vhc_eligible=True,
             )
             vtools.append(vtool)
         return vtools
